@@ -1,4 +1,4 @@
-/*	$NetBSD: extern.h,v 1.4 2006/09/09 16:22:37 manu Exp $	*/
+/*	$NetBSD: extern.h,v 1.5 2009/03/06 11:45:03 tteras Exp $	*/
 
 
 
@@ -14,6 +14,9 @@ int yyparse __P((void));
 void yyfatal __P((const char *));
 void yyerror __P((const char *));
 
+u_int32_t *sendkeymsg_spigrep __P((unsigned int, struct addrinfo *,
+				   struct addrinfo *, int *));
+
 extern int f_rfcmode;
 extern int lineno;
 extern int last_msg_type;
